Imports System.IO
Imports System.Collections

Public Module CoreModule
    Dim R As New System.Random(CInt(Microsoft.VisualBasic.Timer))
    Public Function D(ByVal Roll As Integer) As Integer
        Return R.Next(0, Roll) + 1
    End Function

    Public Function _1D6() As Integer
        Return D(6)
    End Function

    Public Function _2D6() As Integer
        Return D(6) + D(6)
    End Function

    Public Function _3D6() As Integer
        Return D(6) + D(6) + D(6)
    End Function

    Public Function _4D6() As Integer
        Return D(6) + D(6) + D(6) + D(6)
    End Function

    Public Function CRLF() As String
        Return Microsoft.VisualBasic.ControlChars.CrLf
    End Function
End Module

Public Class State
    Private M As New MemoryStream
    Public Property Stream() As MemoryStream
        Get
            Return M
        End Get
        Set(ByVal value As MemoryStream)
            M = value
        End Set
    End Property
End Class


Public MustInherit Class BaseObject(Of T As {BaseObject(Of T), New})
    Public MustOverride Sub Read(ByVal File As BinaryReader)
    Public MustOverride Sub Store(ByVal File As BinaryWriter)
    Protected MustOverride Function Version() As Double

    Public Overridable Sub Assign(ByVal Source As T)
        Me.State = Source.State
    End Sub

    Public Overridable Function Clone() As T
        Dim NewObject As New T
        NewObject.State = Me.State
        Return NewObject
    End Function

    Public Property State() As State
        Get
            Dim S As New State
            Dim W As New BinaryWriter(S.Stream)
            W.BaseStream.Seek(0, SeekOrigin.Begin)
            Me.Store(W)
            Return S
        End Get
        Set(ByVal S As State)
            Dim R As New BinaryReader(S.Stream)
            R.BaseStream.Seek(0, SeekOrigin.Begin)
            Me.Read(R)
        End Set
    End Property
End Class

Public Class ObjectList(Of L As {ObjectList(Of L, T), New}, T As {BaseObject(Of T), New})
    Inherits CollectionBase

    Public Overloads Function Add() As T
        Dim NewObject As New T
        Me.Add(NewObject)
        Return NewObject
    End Function

    Public Overloads Sub Add(ByVal NewObject As T)
        List.Add(NewObject)
    End Sub

    Protected Overridable Function Version() As Double
        Return 1.0
    End Function

    Public Overridable Sub Assign(ByVal NewObject As L)
        Me.State = NewObject.State
    End Sub

    Public Overridable Function Clone() As L
        Dim NewObject As New L
        NewObject.State = Me.State
        Return NewObject
    End Function

    Public Overridable Sub Read(ByVal File As BinaryReader)
        Dim FileVersion As Double = File.ReadDouble
        Dim Count As Integer = File.ReadInt32
        Dim NewObject As T = Nothing
        Dim I As Integer = 0
        For I = 1 To Count
            Me.Add.Read(File)
        Next I
    End Sub

    Default Public ReadOnly Property Item(ByVal Index As Integer) As T
        Get
            Return CType(Me.List(Index), T)
        End Get
    End Property

    Public Overridable Sub Store(ByVal File As BinaryWriter)
        Dim I As Integer = 0
        File.Write(Me.Version)
        File.Write(Me.Count)
        For I = 0 To Me.Count - 1
            Me(I).Store(File)
        Next
    End Sub

    Public Property State() As State
        Get
            Dim S As New State
            Dim W As New BinaryWriter(S.Stream)
            W.BaseStream.Seek(0, SeekOrigin.Begin)
            Me.Store(W)
            Return S
        End Get
        Set(ByVal S As State)
            Dim R As New BinaryReader(S.Stream)
            R.BaseStream.Seek(0, SeekOrigin.Begin)
            Me.Read(R)
        End Set
    End Property
End Class

Public Interface IInterfaceObject
    'Sub Read(ByVal File As BinaryReader)
    'Sub Store(ByVal File As BinaryWriter)
End Interface

Public Class InterfaceList(Of L As {InterfaceList(Of L, T), New}, T As {IInterfaceObject})
    Inherits CollectionBase

    Public Overloads Sub Add(ByVal NewObject As T)
        List.Add(NewObject)
    End Sub

    Default Public ReadOnly Property Item(ByVal Index As Integer) As T
        Get
            Return CType(Me.List(Index), T)
        End Get
    End Property

End Class
