Imports RSC_CoreModule

Public Class RuinTableItem
    Inherits BaseObject(Of RuinTableItem)
    Implements IRuinTableEntry
    Public Description As String
    Public LoValue As Integer
    Public HiValue As Integer

    Public Sub New()

    End Sub

    Public Sub New(ByVal Description As String)
        Me.Description = Description
    End Sub

    Public Sub New(ByVal Description As String, ByVal LoValue As Integer, ByVal HiValue As Integer)
        Me.Description = Description
        Me.LoValue = LoValue
        Me.HiValue = HiValue
    End Sub

    Public Overrides Sub Read(ByVal File As System.IO.BinaryReader)
        Dim FileVersion As Double = File.ReadDouble
        If FileVersion >= 1 Then
            Me.Description = File.ReadString()
            Me.HiValue = File.ReadInt32
            Me.LoValue = File.ReadInt32
        End If
    End Sub

    Public Overrides Sub Store(ByVal File As System.IO.BinaryWriter)
        File.Write(Version)
        File.Write(Me.Description)
        File.Write(Me.HiValue)
        File.Write(Me.LoValue)
    End Sub

    Protected Overrides Function Version() As Double
        Return 1
    End Function

    Public Function Generate() As String Implements IRuinTableEntry.Generate
        Dim I As Integer = 0
        Dim RolledValue As Integer = 0
        If LoValue > 0 And HiValue > 0 Then
            For I = 1 To LoValue
                RolledValue += D(HiValue)
            Next
            Return CStr(RolledValue) & " " & Me.Description
        Else
            Return Me.Description
        End If
    End Function
End Class

'Public Class RuinTableList(Of L As {RuinTableList(Of L, T), New}, T As {IRuinTableEntry})
'    Inherits InterfaceList(Of RuinTableList(Of L, T), T)

'    Public Overloads Sub Add(ByVal Item As IRuinTableEntry)
'        List.Add(Item)
'    End Sub
'End Class

Public Class RuinTable
    Inherits InterfaceList(Of RuinTable, IRuinTableEntry)
    Implements IRuinTableEntry

    Public Overloads Function Add() As RuinTableItem
        Return New RuinTableItem
    End Function

    Public Overloads Sub Add(ByVal ParamArray Items() As String)
        Dim I As Integer
        For I = 0 To UBound(Items, 1)
            Me.Add(New RuinTableItem(Items(I)))
        Next
    End Sub

    Public Function Generate() As String Implements IRuinTableEntry.Generate
        Dim Roll As Integer = D(Me.Count) - 1
        If Me.Count = 0 Then Return ""
        Return Me(Roll).Generate
    End Function
End Class

Public Class RandomRuins
    Public RuinClass As New RuinTable
    Public Condition As New RuinTable
    Public Covering As New RuinTable
    Public RuinState As New RuinTable
    Public Keeper As New RuinTable

    Public Sub New()
        SetupClass()
        SetupCondition()
        SetupCovering()
        SetupState()
        SetupKeeper()
    End Sub

    Public Function Generate() As String
        Dim SB As New System.Text.StringBuilder
        Dim TempS As String
        Dim PuralString As String
        TempS = Me.Condition.Generate
        SB.Append(TempS)
        SB.Append(", ")
        If InStr(TempS, "Covered") > 0 Then
            SB.Append(" by ")
            SB.Append(Me.Covering.Generate)
            SB.Append(" ")
        End If
        TempS = Me.RuinClass.Generate
        If Right$(TempS, 1) <> "s" Then
            SB.Append("a ")
        End If
        SB.Append(Me.RuinState.Generate)
        SB.Append(" ")
        SB.Append(TempS)
        If Right$(TempS, 1) = "s" Then
            PuralString = " are "
        Else
            PuralString = " is "
        End If
        TempS = Me.Keeper.Generate
        If TempS <> "" Then
            SB.Append(PuralString)
            SB.Append("guarded by")
            If Right$(TempS, 1) = "s" Then
                SB.Append(" ")
            Else
                SB.Append(" a ")
            End If
            SB.Append(TempS)
        End If
        Return SB.ToString
    End Function

    Private Sub SetupClass()
        SetupRuins()
        SetupRelics()
        SetupRemains()
        SetupVestiges()
        SetupRemnants()
        SetupRefuse()
        SetupWrecks()
        SetupSkeletons()
        SetupAntiques()
        SetupArtifacts()
    End Sub

    Private Sub SetupCondition()
        Condition.Add(New RuinTableItem("Partially Covered"))
        Condition.Add(New RuinTableItem("Fully Covered"))
        Condition.Add(New RuinTableItem("Above Ground"))
        Condition.Add(New RuinTableItem("On a Rocky Slope"))
        Condition.Add(New RuinTableItem("Inside a Cave"))
        Condition.Add(New RuinTableItem("In a Crevice"))
        Condition.Add(New RuinTableItem("Beneath an Overhang"))
        Condition.Add(New RuinTableItem("In a Large Crater"))
        Condition.Add(New RuinTableItem("Partially Sunken"))
        Condition.Add(New RuinTableItem("Charred & Burnt"))
    End Sub

    Private Sub SetupCovering()
        Covering.Add(New RuinTableItem("Sand"))
        Covering.Add(New RuinTableItem("Ashes"))
        Covering.Add(New RuinTableItem("Cinders"))
        Covering.Add(New RuinTableItem("Earth"))
        Covering.Add(New RuinTableItem("Thickets"))
        Covering.Add(New RuinTableItem("Mold"))
        Covering.Add(New RuinTableItem("Slime"))
        Covering.Add(New RuinTableItem("Rocks"))
        Covering.Add(New RuinTableItem("Webs & Dust"))
        Covering.Add(New RuinTableItem("Vines"))
    End Sub

    Private Sub SetupState()
        RuinState.Add(New RuinTableItem("Crumbled & Decayed"))
        RuinState.Add(New RuinTableItem("Disfigured & Defaced"))
        RuinState.Add(New RuinTableItem("Wormeaten"))
        RuinState.Add(New RuinTableItem("Crystallized & Petrified"))
        RuinState.Add(New RuinTableItem("Corroded & Eroded"))
        RuinState.Add(New RuinTableItem("Collapsed & Tumbled"))
        RuinState.Add(New RuinTableItem("Moldy & Contaminated"))
        RuinState.Add(New RuinTableItem("Dangerously Operational"))
        RuinState.Add(New RuinTableItem("Partially Operational"))
        RuinState.Add(New RuinTableItem("Fully Operational"))
    End Sub

    Private Sub SetupKeeper()
        Keeper.Add(New RuinTableItem("Constructs"))
        Keeper.Add(New RuinTableItem("Giant Class"))
        Keeper.Add(New RuinTableItem("Dragon Class"))
        Keeper.Add(New RuinTableItem("Undead Class"))
        Keeper.Add(New RuinTableItem("Lycanthropes"))
        Keeper.Add(New RuinTableItem("True Giants"))
        Keeper.Add(New RuinTableItem("Animals"))
        Keeper.Add(New RuinTableItem("Insects"))
        Keeper.Add(New RuinTableItem("Trap"))
        Keeper.Add(New RuinTableItem(""))
    End Sub

    Private Sub SetupRuins()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        RuinClass.Add(Table)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SetupManor(SubTable)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SetupVillage(SubTable)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SetupCity(SubTable)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SetupCitadel(SubTable)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SetupCastle(SubTable)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SetupTemple(SubTable)

    End Sub

    Private Sub SetupManor(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("Hut"))
        Table.Add(New RuinTableItem("Hovel"))
        Table.Add(New RuinTableItem("Hall"))
        Table.Add(New RuinTableItem("Villa"))
        Table.Add(New RuinTableItem("Cottage"))
        Table.Add(New RuinTableItem("Palace"))
    End Sub

    Private Sub SetupVillage(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("Huts", 2, 6))
        Table.Add(New RuinTableItem("Hovels", 4, 6))
        Table.Add(New RuinTableItem("Cottages", 6, 6))
        Table.Add(New RuinTableItem("Cottages surrounded by a Ditch", 6, 6))
        Table.Add(New RuinTableItem("Cottages surrounded by a Pallisade", 6, 6))
        Table.Add(New RuinTableItem("Cottages surrounded by a Pallisade and a Moat", 6, 6))
    End Sub

    Private Sub SetupCity(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("Huts and a Citadel", 7, 6))
        Table.Add(New RuinTableItem("Houses", 8, 6))
        Table.Add(New RuinTableItem("Houses surrounded by a Wall", 9, 6))
        Table.Add(New RuinTableItem("Houses and a Citadel", 9, 6))
        Table.Add(New RuinTableItem("Houses", 12, 10))
        Table.Add(New RuinTableItem("Houses and a Temple", 12, 10))
    End Sub

    Private Sub SetupCitadel(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("a Tower"))
        Table.Add(New RuinTableItem("a Tower surrounded by a Wall"))
        Table.Add(New RuinTableItem("a Great Keep"))
        Table.Add(New RuinTableItem("a Keep with four towers"))
        Table.Add(New RuinTableItem("a Keep with four Towers, surrounded by a Wall"))
        Table.Add(New RuinTableItem("a Keep with four Towers, surrounded by a Moat and a Wall"))

    End Sub

    Private Sub SetupCastle(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("a Keep surrounded by a Pallisade"))
        Table.Add(New RuinTableItem("a Keep surrounded by a Moat and a Pallisade"))
        Table.Add(New RuinTableItem("a Keep with a Manor, surrounded by a Moat and a Pallisade"))
        Table.Add(New RuinTableItem("a Keep with a Manor, surrounded by a Moat and Walls"))
        Table.Add(New RuinTableItem("a Keep with four towers, surrounded by a Moat, and a Pallisade"))
        Table.Add(New RuinTableItem("a Keep with four towers, surrounded by a Moat, and Walls"))
    End Sub

    Private Sub SetupTemple(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("Altar"))
        Table.Add(New RuinTableItem("Shrine"))
        Table.Add(New RuinTableItem("Sanctuary"))
        Table.Add(New RuinTableItem("Oracle"))
        Table.Add(New RuinTableItem("Pantheon"))
        Table.Add(New RuinTableItem("Monastery"))
    End Sub

    Private Sub SetupRelics()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        RuinClass.Add(Table)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SetupTools(SubTable)
        SetupMachines(SubTable)
        SetupTombs(SubTable)
        SetupArmor(SubTable)
        SetupWeapons(SubTable)
        SetupContainers(SubTable)
    End Sub

    Private Sub SetupTools(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("Ladder"))
        Table.Add(New RuinTableItem("Plow"))
        Table.Add(New RuinTableItem("Pick"))
        Table.Add(New RuinTableItem("Hoe"))
        Table.Add(New RuinTableItem("Anvil"))
        Table.Add(New RuinTableItem("Axe"))
    End Sub

    Private Sub SetupMachines(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("Loom"))
        Table.Add(New RuinTableItem("Grinding Wheel"))
        Table.Add(New RuinTableItem("Clock"))
        Table.Add(New RuinTableItem("Balance"))
        Table.Add(New RuinTableItem("Potter's Wheel"))
        Table.Add(New RuinTableItem("Press"))

    End Sub

    Private Sub SetupTombs(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("Grave"))
        Table.Add(New RuinTableItem("Sepulcher"))
        Table.Add(New RuinTableItem("Mausoleum"))
        Table.Add(New RuinTableItem("Catacombs"))
        Table.Add(New RuinTableItem("Vault"))
        Table.Add(New RuinTableItem("Crypt"))

    End Sub

    Private Sub SetupArmor(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("BreastPlate"))
        Table.Add(New RuinTableItem("Greaves"))
        Table.Add(New RuinTableItem("Gauntlets"))
        Table.Add(New RuinTableItem("Helmet"))
        Table.Add(New RuinTableItem("Chainmail"))
        Table.Add(New RuinTableItem("Shield"))

    End Sub

    Private Sub SetupWeapons(ByVal Table As RuinTable)
        Table.Add(New RuinTableItem("Dagger"))
        Table.Add(New RuinTableItem("Scimitar"))
        Table.Add(New RuinTableItem("Hand Axe"))
        Table.Add(New RuinTableItem("Spear"))
        Table.Add(New RuinTableItem("Trident"))
        Table.Add(New RuinTableItem("Battleaxe"))

    End Sub

    Private Sub SetupContainers(ByVal Table As RuinTable)

        Table.Add(New RuinTableItem("Barrels"))
        Table.Add(New RuinTableItem("Urns"))
        Table.Add(New RuinTableItem("Trunks"))
        Table.Add(New RuinTableItem("Jars"))
        Table.Add(New RuinTableItem("Bottles"))
        Table.Add(New RuinTableItem("Boxes"))
    End Sub

    Private Sub SetupRemains()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        RuinClass.Add(Table)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Eating Utensil", "Digging Implement", "Writing Implement", "Navigating Implement", "Measuring Implement", "Musical Implement")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Hauberk", "Boots", "Cloak", "Tunic", "Mask", "Breeches")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Swimming Harness", "Flying Harness", "Giant Animal Harness", "Small Animal Harness", "Medium Animal Harness", "Colossal Animal Harness")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Doll", "Toy Vehicle", "Toy Weapon", "Toy Tool", "Game", "Toy House")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Monocle", "Spectacles", "Spyglass", "Mirror", "Colored Pane of Glass", "Periscope")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Lexicon", "Scroll", "Manual", "Tablet", "Book", "Codex")

    End Sub

    Private Sub SetupVestiges()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        RuinClass.Add(Table)


        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Pyramid used as a Tomb", "Pyramid used as a Temple", "Pyramid used as an Observatory", "Pyramid used as a Palace")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Stepped Pyramid used as a Tomb", "Stepped Pyramid used as a Temple", "Stepped Pyramid used as an Observatory", "Stepped Pyramid used as a Palace")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Stepped Pyramid used as a Tomb", "Stepped Pyramid used as a Temple", "Stepped Pyramid used as an Observatory", "Stepped Pyramid used as a Palace")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Sacrificial Mound", "Burial Mound", "Treasure Mound", "a Lair in a Mound", "Sacred Mound", "a Polymorphed Mound")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Giant Fountain", "a fountain with statues", "a minature fountain", "a geometric fountain", "a ""Make a Wish"" fountain", "a fountain with spray jets")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("a totem with carved mammals", "a totem with carved humans", "a totem with carved god(s)", "a totem with carved monsters", "a totem with carved birds", "a totem with carved snakes")


        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("a one foot square unbarred opening to a sewer", "a two foot square unbarred opening to a sewer", "a one foot square locked opening to a sewer", "a two foot square locked opening to a sewer", "a four foot square locked opening to a sewer", "a eight foot square locked opening to a sewer")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("column", "hewn statue", "minaret", "obelisk", "effigy", "monument")
    End Sub

    Private Sub SetupRemnants()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        RuinClass.Add(Table)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Track", "Trail", "Gravel Road", "Roman Paved Road", "Asphalt Road", "Concrete Road")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("a tombstone with a plaque", "a cairn", "a staff marking a burial spot", "beacon marking a burial spot", "a pyre marking a burial spot", "a stone pile marking a burial spot")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("a guiding signpost", "a signpost with a omen written on it", "a trade signpost", "a warning signpost", "a signpost with a emblem marked on it", "a boundary signpost")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("tunnel", "a pipe", "a well", "a passage", "a canal", "a aqueduct")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("mud brick masonry", "stone brick masonry", "marble block masonry", "plaster masonry", "wattle masonary", "stone block masonry")


        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("a rope bridge", "a wooden bridge", "a stone bridge", "a earth bridge", "a natural bridge", "a brick bridge")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("carved cliffside", "sculptured mound", "colossal statue", "palace", "mill", "calendar stone")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Arnenal works", "Granery works", "Paved Plaza", "Viaduct", "Reservoir", "Cistern")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("stairway", "ramp", "shaft", "spire", "roof", "tunnel")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("stockade wall", "barricade", "fence", "wall rampart", "dike", "partition")
    End Sub

    Private Sub SetupRefuse()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        RuinClass.Add(Table)

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Viscera", "Bones", "Gore", "Gristle", "Fat", "Talons")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Sewage with soap", "sewage with body wastes", "sewage with oils", "slop sewage", "sewage with chemicals", "sewage with lint")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Buckle", "Lacing", "Crossbar", "Pommels", "Arrowhead", "Spike")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("leather scraps", "papyrus scraps", "a handle", "shield boss", "pole", "linen scraps")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("food fat", "fruit", "seeds", "vegetables", "minerals", "meat")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("wood", "coal", "peat", "dung", "petroleum", "wax")
    End Sub

    Private Sub SetupWrecks()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        Dim ST2 As RuinTable
        RuinClass.Add(Table)


        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("raft", "canoe", "pig bladder", "longboat", "merchant ship", "man o' war")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("Sledge", "wagon", "a litter", "carriage", "wain", "chariot")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("balloon", "wings", "hang-glider", "parachute", "roc carriage")
        ST2 = New RuinTable
        SubTable.Add(ST2)
        ST2.Add("Prop driven aircraft", "jet aircraft", "reentry capsule", "helicopter", "anti-grav aircraft")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("onager", "screw war engine", "ram", "siege tower", "springal", "trebuchet")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("dolphin sled", "seahorse carriage", "giant turtle house", "diving bell", "pocket submarine", "nuclear submarine")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("mole sled", "rock borer", "mine cart", "earth borer", "worm saddle", "rat chariot")
    End Sub

    Private Sub SetupSkeletons()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        RuinClass.Add(Table)


        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("miniscule skeleton", "halfling skeleton", "goblin skeleton", "pixie skeleton", "gnome skeleton", "kobold skeleton")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("human skeleton", "elven skeleton", "orc skeleton", "dwarf skeleton", "lizardman skeleton", "hobgoblin skeleton")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("ogre skeleton", "troll skeleton", "true giant skeleton", "dinosaur skeleton", "sea monster skeleton", "whale skeleton")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("cubic skeleton", "crystalline skeleton", "multi-limbed skeleton", "multi-headed skeleton", "winged skeleton", "armor-plated skeleton")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("miniscule skull", "medium skull", "giant skull", "colossal skull", "multi-horned skull", "multiple eye sockets skull")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("colossal humaniod skeleton", "colossal avian skeleton", "colossal reptilian skeleton", "colossal ursiod skeleton", "colossal amphibian skeleton", "colossal crustacean skeleton")
    End Sub

    Private Sub SetupAntiques()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        RuinClass.Add(Table)


        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("miniature statue", "half size statue", "life size statue", "giant size statue", "abstract statue", "magic statue")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("throne", "chest", "giant size furniture", "miniature furniture", "stone seat", "stone table")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("an engraving of a battle scene", "an engraving of a coronation", "an engraving of a punishment", "an religous themed engraving", "a romantic themed engraving", "an engraving of a curse")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("stone idol(s)", "plaster idol(s)", "metal idol(s)", "wooden idol(s)")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("faucet", "lamp", "bell", "fresco", "hinges", "knocker")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("basket", "vase", "minature painting", "abacas", "ship's figurehead", "bust")
    End Sub

    Private Sub SetupArtifacts()
        Dim Table As New RuinTable
        Dim SubTable As RuinTable
        RuinClass.Add(Table)


        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("sword", "dagger", "hammer", "club", "battleaxe", "javelin")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("animated entertainment", "musical entertainment", "dancing entertainment", "serving entertainment", "intensifying entertainment", "dreaming entertainment")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("machine protective device", "protective staff", "a protective vial", "protective garment", "protective talisman", "protective armor")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("informational stone", "informational flask", "informational orb", "informational diadem", "informational crystal ball", "informational necklace")

        SubTable = New RuinTable
        Table.Add(SubTable)
        SubTable.Add("ring of leadership", "gem of leadership", "throne of leadership", "rod of leadership", "sword of leadership", "scepter of leadership")
    End Sub
End Class

Public Interface IRuinTableEntry
    Inherits IInterfaceObject
    Function Generate() As String
End Interface

